@@ -2,6 +2,7 @@
 use ruff_benchmark::criterion;
 use ruff_benchmark::real_world_projects::{InstalledProject, RealWorldProject};
 
+use std::fmt::Write;
 use std::ops::Range;
 
 use criterion::{BatchSize, Criterion, criterion_group, criterion_main};
@@ -441,6 +442,37 @@ fn benchmark_complex_constrained_attributes_2(criterion: &mut Criterion) {
     });
 }
 
+fn benchmark_many_enum_members(criterion: &mut Criterion) {
+    const NUM_ENUM_MEMBERS: usize = 512;
+
+    setup_rayon();
+
+    let mut code = String::new();
+    writeln!(&mut code, "from enum import Enum").ok();
+
+    writeln!(&mut code, "class E(Enum):").ok();
+    for i in 0..NUM_ENUM_MEMBERS {
+        writeln!(&mut code, "    m{i} = {i}").ok();
+    }
+    writeln!(&mut code).ok();
+
+    for i in 0..NUM_ENUM_MEMBERS {
+        writeln!(&mut code, "print(E.m{i})").ok();
+    }
+
+    criterion.bench_function("ty_micro[many_enum_members]", |b| {
+        b.iter_batched_ref(
+            || setup_micro_case(&code),
+            |case| {
+                let Case { db, .. } = case;
+                let result = db.check();
+                assert_eq!(result.len(), 0);
+            },
+            BatchSize::SmallInput,
+        );
+    });
+}
+
 struct ProjectBenchmark<'a> {
     project: InstalledProject<'a>,
     fs: MemoryFileSystem,
@@ -498,11 +530,8 @@ fn bench_project(benchmark: &ProjectBenchmark, criterion: &mut Criterion) {
         let diagnostics = result.len();
 
         assert!(
-            diagnostics > 1 && diagnostics <= max_diagnostics,
-            "Expected between {} and {} diagnostics but got {}",
-            1,
-            max_diagnostics,
-            diagnostics
+            diagnostics <= max_diagnostics,
+            "Expected <={max_diagnostics} diagnostics but got {diagnostics}"
         );
     }
 
@@ -570,6 +599,23 @@ fn anyio(criterion: &mut Criterion) {
     bench_project(&benchmark, criterion);
 }
 
+fn datetype(criterion: &mut Criterion) {
+    let benchmark = ProjectBenchmark::new(
+        RealWorldProject {
+            name: "DateType",
+            repository: "https://github.com/glyph/DateType",
+            commit: "57c9c93cf2468069f72945fc04bf27b64100dad8",
+            paths: vec![SystemPath::new("src")],
+            dependencies: vec![],
+            max_dep_date: "2025-07-04",
+            python_version: PythonVersion::PY313,
+        },
+        0,
+    );
+
+    bench_project(&benchmark, criterion);
+}
+
 criterion_group!(check_file, benchmark_cold, benchmark_incremental);
 criterion_group!(
     micro,
@@ -577,6 +623,7 @@ criterion_group!(
     benchmark_many_tuple_assignments,
     benchmark_complex_constrained_attributes_1,
     benchmark_complex_constrained_attributes_2,
+    benchmark_many_enum_members,
 );
-criterion_group!(project, anyio, attrs, hydra);
+criterion_group!(project, anyio, attrs, hydra, datetype);
 criterion_main!(check_file, micro, project);
