@@ -1,5 +1,4 @@
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::helpers::ReturnStatementVisitor;
 use ruff_python_ast::identifier::Identifier;
 use ruff_python_ast::visitor::Visitor;
@@ -9,11 +8,12 @@ use ruff_python_semantic::analyze::terminal::Terminal;
 use ruff_python_semantic::analyze::type_inference::{NumberLike, PythonType, ResolvedPythonType};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
-/// Checks for `__len__` implementations that return values other than a non-negative
-/// integer.
+/// Checks for `__len__` implementations that return values that are not non-negative
+/// integers.
 ///
 /// ## Why is this bad?
 /// The `__len__` method should return a non-negative integer. Returning a different
@@ -21,7 +21,7 @@ use crate::checkers::ast::Checker;
 ///
 /// Note: `bool` is a subclass of `int`, so it's technically valid for `__len__` to
 /// return `True` or `False`. However, for consistency with other rules, Ruff will
-/// still raise when `__len__` returns a `bool`.
+/// still emit a diagnostic when `__len__` returns a `bool`.
 ///
 /// ## Example
 /// ```python
@@ -37,21 +37,20 @@ use crate::checkers::ast::Checker;
 ///         return 2
 /// ```
 ///
-///
 /// ## References
 /// - [Python documentation: The `__len__` method](https://docs.python.org/3/reference/datamodel.html#object.__len__)
-#[violation]
-pub struct InvalidLengthReturnType;
+#[derive(ViolationMetadata)]
+pub(crate) struct InvalidLengthReturnType;
 
 impl Violation for InvalidLengthReturnType {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`__len__` does not return a non-negative integer")
+        "`__len__` does not return a non-negative integer".to_string()
     }
 }
 
-/// E0303
-pub(crate) fn invalid_length_return(checker: &mut Checker, function_def: &ast::StmtFunctionDef) {
+/// PLE0303
+pub(crate) fn invalid_length_return(checker: &Checker, function_def: &ast::StmtFunctionDef) {
     if function_def.name.as_str() != "__len__" {
         return;
     }
@@ -74,10 +73,7 @@ pub(crate) fn invalid_length_return(checker: &mut Checker, function_def: &ast::S
 
     // If there are no return statements, add a diagnostic.
     if terminal == Terminal::Implicit {
-        checker.diagnostics.push(Diagnostic::new(
-            InvalidLengthReturnType,
-            function_def.identifier(),
-        ));
+        checker.report_diagnostic(InvalidLengthReturnType, function_def.identifier());
         return;
     }
 
@@ -96,15 +92,11 @@ pub(crate) fn invalid_length_return(checker: &mut Checker, function_def: &ast::S
                         | ResolvedPythonType::Atom(PythonType::Number(NumberLike::Integer))
                 )
             {
-                checker
-                    .diagnostics
-                    .push(Diagnostic::new(InvalidLengthReturnType, value.range()));
+                checker.report_diagnostic(InvalidLengthReturnType, value.range());
             }
         } else {
             // Disallow implicit `None`.
-            checker
-                .diagnostics
-                .push(Diagnostic::new(InvalidLengthReturnType, stmt.range()));
+            checker.report_diagnostic(InvalidLengthReturnType, stmt.range());
         }
     }
 }
