@@ -1,8 +1,10 @@
+use ruff_macros::{ViolationMetadata, derive_message_formats};
+use ruff_python_ast::identifier::Identifier;
 use ruff_python_ast::{self as ast, ExceptHandler, Stmt};
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
-use ruff_python_ast::identifier::Identifier;
+use crate::Violation;
+
+use crate::checkers::ast::Checker;
 
 /// ## What it does
 /// Checks for functions or methods with too many statements.
@@ -45,8 +47,8 @@ use ruff_python_ast::identifier::Identifier;
 ///
 /// ## Options
 /// - `lint.pylint.max-statements`
-#[violation]
-pub struct TooManyStatements {
+#[derive(ViolationMetadata)]
+pub(crate) struct TooManyStatements {
     statements: usize,
     max_statements: usize,
 }
@@ -137,31 +139,36 @@ fn num_statements(stmts: &[Stmt]) -> usize {
 
 /// PLR0915
 pub(crate) fn too_many_statements(
+    checker: &Checker,
     stmt: &Stmt,
     body: &[Stmt],
     max_statements: usize,
-) -> Option<Diagnostic> {
+) {
     let statements = num_statements(body);
     if statements > max_statements {
-        Some(Diagnostic::new(
+        checker.report_diagnostic(
             TooManyStatements {
                 statements,
                 max_statements,
             },
             stmt.identifier(),
-        ))
-    } else {
-        None
+        );
     }
 }
 
 #[cfg(test)]
 mod tests {
     use anyhow::Result;
-    use ruff_python_parser::parse_suite;
+
+    use ruff_python_ast::Suite;
+    use ruff_python_parser::parse_module;
 
     use super::num_statements;
 
+    fn parse_suite(source: &str) -> Result<Suite> {
+        Ok(parse_module(source)?.into_suite())
+    }
+
     #[test]
     fn pass() -> Result<()> {
         let source: &str = r"
