@@ -1,13 +1,14 @@
 use anyhow::Result;
 
-use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, Keyword};
 use ruff_python_semantic::Modules;
+use ruff_python_trivia::CommentRanges;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
-use crate::fix::edits::{remove_argument, Parentheses};
+use crate::fix::edits::{Parentheses, remove_argument};
+use crate::{Edit, Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for uses of `subprocess.run` that send `stdout` and `stderr` to a
@@ -33,18 +34,24 @@ use crate::fix::edits::{remove_argument, Parentheses};
 /// subprocess.run(["foo"], capture_output=True)
 /// ```
 ///
+/// ## Fix safety
+///
+/// This rule's fix is marked as unsafe because replacing `stdout=subprocess.PIPE` and
+/// `stderr=subprocess.PIPE` with `capture_output=True` may delete comments attached
+/// to the original arguments.
+///
 /// ## References
 /// - [Python 3.7 release notes](https://docs.python.org/3/whatsnew/3.7.html#subprocess)
 /// - [Python documentation: `subprocess.run`](https://docs.python.org/3/library/subprocess.html#subprocess.run)
-#[violation]
-pub struct ReplaceStdoutStderr;
+#[derive(ViolationMetadata)]
+pub(crate) struct ReplaceStdoutStderr;
 
 impl Violation for ReplaceStdoutStderr {
     const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
 
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Prefer `capture_output` over sending `stdout` and `stderr` to `PIPE`")
+        "Prefer `capture_output` over sending `stdout` and `stderr` to `PIPE`".to_string()
     }
 
     fn fix_title(&self) -> Option<String> {
@@ -53,7 +60,7 @@ impl Violation for ReplaceStdoutStderr {
 }
 
 /// UP022
-pub(crate) fn replace_stdout_stderr(checker: &mut Checker, call: &ast::ExprCall) {
+pub(crate) fn replace_stdout_stderr(checker: &Checker, call: &ast::ExprCall) {
     if !checker.semantic().seen_module(Modules::SUBPROCESS) {
         return;
     }
@@ -88,12 +95,18 @@ pub(crate) fn replace_stdout_stderr(checker: &mut Checker, call: &ast::ExprCall)
             return;
         }
 
-        let mut diagnostic = Diagnostic::new(ReplaceStdoutStderr, call.range());
+        let mut diagnostic = checker.report_diagnostic(ReplaceStdoutStderr, call.range());
         if call.arguments.find_keyword("capture_output").is_none() {
-            diagnostic
-                .try_set_fix(|| generate_fix(stdout, stderr, call, checker.locator().contents()));
+            diagnostic.try_set_fix(|| {
+                generate_fix(
+                    stdout,
+                    stderr,
+                    call,
+                    checker.locator().contents(),
+                    checker.comment_ranges(),
+                )
+            });
         }
-        checker.diagnostics.push(diagnostic);
     }
 }
 
@@ -103,6 +116,7 @@ fn generate_fix(
     stderr: &Keyword,
     call: &ast::ExprCall,
     source: &str,
+    comment_ranges: &CommentRanges,
 ) -> Result<Fix> {
     let (first, second) = if stdout.start() < stderr.start() {
         (stdout, stderr)
@@ -117,6 +131,7 @@ fn generate_fix(
             &call.arguments,
             Parentheses::Preserve,
             source,
+            comment_ranges,
         )?],
     ))
 }
