@@ -1,17 +1,19 @@
-use crate::comments::Comments;
-use crate::other::f_string_element::FStringExpressionElementContext;
-use crate::PyFormatOptions;
-use ruff_formatter::{Buffer, FormatContext, GroupId, IndentWidth, SourceCode};
-use ruff_python_ast::str::Quote;
-use ruff_source_file::Locator;
 use std::fmt::{Debug, Formatter};
 use std::ops::{Deref, DerefMut};
 
-#[derive(Clone)]
+use ruff_formatter::{Buffer, FormatContext, GroupId, IndentWidth, SourceCode};
+use ruff_python_ast::str::Quote;
+use ruff_python_parser::Tokens;
+
+use crate::PyFormatOptions;
+use crate::comments::Comments;
+use crate::other::interpolated_string::InterpolatedStringContext;
+
 pub struct PyFormatContext<'a> {
     options: PyFormatOptions,
     contents: &'a str,
     comments: Comments<'a>,
+    tokens: &'a Tokens,
     node_level: NodeLevel,
     indent_level: IndentLevel,
     /// Set to a non-None value when the formatter is running on a code
@@ -23,20 +25,26 @@ pub struct PyFormatContext<'a> {
     /// quote style that is inverted from the one here in order to ensure that
     /// the formatted Python code will be valid.
     docstring: Option<Quote>,
-    /// The state of the formatter with respect to f-strings.
-    f_string_state: FStringState,
+    /// The state of the formatter with respect to f-strings and t-strings.
+    interpolated_string_state: InterpolatedStringState,
 }
 
 impl<'a> PyFormatContext<'a> {
-    pub(crate) fn new(options: PyFormatOptions, contents: &'a str, comments: Comments<'a>) -> Self {
+    pub(crate) fn new(
+        options: PyFormatOptions,
+        contents: &'a str,
+        comments: Comments<'a>,
+        tokens: &'a Tokens,
+    ) -> Self {
         Self {
             options,
             contents,
             comments,
+            tokens,
             node_level: NodeLevel::TopLevel(TopLevelStatementPosition::Other),
             indent_level: IndentLevel::new(0),
             docstring: None,
-            f_string_state: FStringState::Outside,
+            interpolated_string_state: InterpolatedStringState::Outside,
         }
     }
 
@@ -44,11 +52,6 @@ impl<'a> PyFormatContext<'a> {
         self.contents
     }
 
-    #[allow(unused)]
-    pub(crate) fn locator(&self) -> Locator<'a> {
-        Locator::new(self.contents)
-    }
-
     pub(crate) fn set_node_level(&mut self, level: NodeLevel) {
         self.node_level = level;
     }
@@ -69,6 +72,10 @@ impl<'a> PyFormatContext<'a> {
         &self.comments
     }
 
+    pub(crate) fn tokens(&self) -> &'a Tokens {
+        self.tokens
+    }
+
     /// Returns a non-None value only if the formatter is running on a code
     /// snippet within a docstring.
     ///
@@ -90,16 +97,18 @@ impl<'a> PyFormatContext<'a> {
         }
     }
 
-    pub(crate) fn f_string_state(&self) -> FStringState {
-        self.f_string_state
+    pub(crate) fn interpolated_string_state(&self) -> InterpolatedStringState {
+        self.interpolated_string_state
     }
 
-    pub(crate) fn set_f_string_state(&mut self, f_string_state: FStringState) {
-        self.f_string_state = f_string_state;
+    pub(crate) fn set_interpolated_string_state(
+        &mut self,
+        interpolated_string_state: InterpolatedStringState,
+    ) {
+        self.interpolated_string_state = interpolated_string_state;
     }
 
     /// Returns `true` if preview mode is enabled.
-    #[allow(unused)]
     pub(crate) const fn is_preview(&self) -> bool {
         self.options.preview().is_enabled()
     }
@@ -129,17 +138,28 @@ impl Debug for PyFormatContext<'_> {
 }
 
 #[derive(Clone, Copy, Debug, Default)]
-pub(crate) enum FStringState {
+pub(crate) enum InterpolatedStringState {
     /// The formatter is inside an f-string expression element i.e., between the
     /// curly brace in `f"foo {x}"`.
     ///
     /// The containing `FStringContext` is the surrounding f-string context.
-    InsideExpressionElement(FStringExpressionElementContext),
+    InsideInterpolatedElement(InterpolatedStringContext),
     /// The formatter is outside an f-string.
     #[default]
     Outside,
 }
 
+impl InterpolatedStringState {
+    pub(crate) fn can_contain_line_breaks(self) -> Option<bool> {
+        match self {
+            InterpolatedStringState::InsideInterpolatedElement(context) => {
+                Some(context.is_multiline())
+            }
+            InterpolatedStringState::Outside => None,
+        }
+    }
+}
+
 /// The position of a top-level statement in the module.
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Default)]
 pub(crate) enum TopLevelStatementPosition {
@@ -214,7 +234,7 @@ where
     }
 }
 
-impl<'ast, 'buf, B> Deref for WithNodeLevel<'ast, 'buf, B>
+impl<'ast, B> Deref for WithNodeLevel<'ast, '_, B>
 where
     B: Buffer<Context = PyFormatContext<'ast>>,
 {
@@ -225,7 +245,7 @@ where
     }
 }
 
-impl<'ast, 'buf, B> DerefMut for WithNodeLevel<'ast, 'buf, B>
+impl<'ast, B> DerefMut for WithNodeLevel<'ast, '_, B>
 where
     B: Buffer<Context = PyFormatContext<'ast>>,
 {
@@ -358,25 +378,25 @@ where
     }
 }
 
-pub(crate) struct WithFStringState<'a, B, D>
+pub(crate) struct WithInterpolatedStringState<'a, B, D>
 where
     D: DerefMut<Target = B>,
     B: Buffer<Context = PyFormatContext<'a>>,
 {
     buffer: D,
-    saved_location: FStringState,
+    saved_location: InterpolatedStringState,
 }
 
-impl<'a, B, D> WithFStringState<'a, B, D>
+impl<'a, B, D> WithInterpolatedStringState<'a, B, D>
 where
     D: DerefMut<Target = B>,
     B: Buffer<Context = PyFormatContext<'a>>,
 {
-    pub(crate) fn new(expr_location: FStringState, mut buffer: D) -> Self {
+    pub(crate) fn new(expr_location: InterpolatedStringState, mut buffer: D) -> Self {
         let context = buffer.state_mut().context_mut();
-        let saved_location = context.f_string_state();
+        let saved_location = context.interpolated_string_state();
 
-        context.set_f_string_state(expr_location);
+        context.set_interpolated_string_state(expr_location);
 
         Self {
             buffer,
@@ -385,7 +405,7 @@ where
     }
 }
 
-impl<'a, B, D> Deref for WithFStringState<'a, B, D>
+impl<'a, B, D> Deref for WithInterpolatedStringState<'a, B, D>
 where
     D: DerefMut<Target = B>,
     B: Buffer<Context = PyFormatContext<'a>>,
@@ -397,7 +417,7 @@ where
     }
 }
 
-impl<'a, B, D> DerefMut for WithFStringState<'a, B, D>
+impl<'a, B, D> DerefMut for WithInterpolatedStringState<'a, B, D>
 where
     D: DerefMut<Target = B>,
     B: Buffer<Context = PyFormatContext<'a>>,
@@ -407,7 +427,7 @@ where
     }
 }
 
-impl<'a, B, D> Drop for WithFStringState<'a, B, D>
+impl<'a, B, D> Drop for WithInterpolatedStringState<'a, B, D>
 where
     D: DerefMut<Target = B>,
     B: Buffer<Context = PyFormatContext<'a>>,
@@ -416,6 +436,6 @@ where
         self.buffer
             .state_mut()
             .context_mut()
-            .set_f_string_state(self.saved_location);
+            .set_interpolated_string_state(self.saved_location);
     }
 }
