@@ -7,6 +7,7 @@ use thiserror::Error;
 use ty_python_semantic::ProgramSettings;
 
 use crate::combine::Combine;
+use crate::metadata::options::ProjectOptionsOverrides;
 use crate::metadata::pyproject::{Project, PyProject, PyProjectError, ResolveRequiresPythonError};
 use crate::metadata::value::ValueSource;
 pub use options::Options;
@@ -276,6 +277,10 @@ impl ProjectMetadata {
             .to_program_settings(self.root(), self.name(), system, vendored)
     }
 
+    pub fn apply_overrides(&mut self, overrides: &ProjectOptionsOverrides) {
+        self.options = overrides.apply_to(std::mem::take(&mut self.options));
+    }
+
     /// Combine the project options with the CLI options where the CLI options take precedence.
     pub fn apply_options(&mut self, options: Options) {
         self.options = options.combine(std::mem::take(&mut self.options));
@@ -455,8 +460,7 @@ mod tests {
   |
 5 |                     [tool.ty
   |                             ^
-invalid table header
-expected `.`, `]`
+unclosed table, expected `]`
 "#,
         );
 
