@@ -192,6 +192,17 @@
 //! for that place that we need for that use or definition. When we reach the end of the scope, it
 //! records the state for each place as the public definitions of that place.
 //!
+//! ```python
+//! x = 1
+//! x = 2
+//! y = x
+//! if flag:
+//!     x = 3
+//! else:
+//!     x = 4
+//! z = x
+//! ```
+//!
 //! Let's walk through the above example. Initially we do not have any record of `x`. When we add
 //! the new place (before we process the first binding), we create a new undefined `PlaceState`
 //! which has a single live binding (the "unbound" definition) and a single live declaration (the
@@ -237,6 +248,7 @@ use self::place_state::{
     LiveDeclarationsIterator, PlaceState, ScopedDefinitionId,
 };
 use crate::node_key::NodeKey;
+use crate::place::BoundnessAnalysis;
 use crate::semantic_index::ast_ids::ScopedUseId;
 use crate::semantic_index::definition::{Definition, DefinitionState};
 use crate::semantic_index::narrowing_constraints::{
@@ -246,18 +258,19 @@ use crate::semantic_index::place::{
     FileScopeId, PlaceExpr, PlaceExprWithFlags, ScopeKind, ScopedPlaceId,
 };
 use crate::semantic_index::predicate::{
-    Predicate, Predicates, PredicatesBuilder, ScopedPredicateId, StarImportPlaceholderPredicate,
+    Predicate, PredicateOrLiteral, Predicates, PredicatesBuilder, ScopedPredicateId,
 };
 use crate::semantic_index::reachability_constraints::{
     ReachabilityConstraints, ReachabilityConstraintsBuilder, ScopedReachabilityConstraintId,
 };
+use crate::semantic_index::use_def::place_state::PreviousDefinitions;
 use crate::semantic_index::{EagerSnapshotResult, SemanticIndex};
 use crate::types::{IntersectionBuilder, Truthiness, Type, infer_narrowing_constraint};
 
 mod place_state;
 
 /// Applicable definitions and constraints for every use of a name.
-#[derive(Debug, PartialEq, Eq, salsa::Update)]
+#[derive(Debug, PartialEq, Eq, salsa::Update, get_size2::GetSize)]
 pub(crate) struct UseDefMap<'db> {
     /// Array of [`Definition`] in this scope. Only the first entry should be [`DefinitionState::Undefined`];
     /// this represents the implicit "unbound"/"undeclared" definition of every place.
@@ -293,10 +306,16 @@ pub(crate) struct UseDefMap<'db> {
     /// If the definition is both a declaration and a binding -- `x: int = 1` for example -- then
     /// we don't actually need anything here, all we'll need to validate is that our own RHS is a
     /// valid assignment to our own annotation.
-    bindings_by_declaration: FxHashMap<Definition<'db>, Bindings>,
+    ///
+    /// If we see a binding to a `Final`-qualified symbol, we also need this map to find previous
+    /// bindings to that symbol. If there are any, the assignment is invalid.
+    bindings_by_definition: FxHashMap<Definition<'db>, Bindings>,
 
     /// [`PlaceState`] visible at end of scope for each place.
-    public_places: IndexVec<ScopedPlaceId, PlaceState>,
+    end_of_scope_places: IndexVec<ScopedPlaceId, PlaceState>,
+
+    /// All potentially reachable bindings and declarations, for each place.
+    reachable_definitions: IndexVec<ScopedPlaceId, ReachableDefinitions>,
 
     /// Snapshot of bindings in this scope that can be used to resolve a reference in a nested
     /// eager scope.
@@ -332,7 +351,10 @@ impl<'db> UseDefMap<'db> {
         &self,
         use_id: ScopedUseId,
     ) -> BindingWithConstraintsIterator<'_, 'db> {
-        self.bindings_iterator(&self.bindings_by_use[use_id])
+        self.bindings_iterator(
+            &self.bindings_by_use[use_id],
+            BoundnessAnalysis::BasedOnUnboundVisibility,
+        )
     }
 
     pub(crate) fn applicable_constraints(
@@ -394,11 +416,24 @@ impl<'db> UseDefMap<'db> {
             .may_be_true()
     }
 
-    pub(crate) fn public_bindings(
+    pub(crate) fn end_of_scope_bindings(
         &self,
         place: ScopedPlaceId,
     ) -> BindingWithConstraintsIterator<'_, 'db> {
-        self.bindings_iterator(self.public_places[place].bindings())
+        self.bindings_iterator(
+            self.end_of_scope_places[place].bindings(),
+            BoundnessAnalysis::BasedOnUnboundVisibility,
+        )
+    }
+
+    pub(crate) fn all_reachable_bindings(
+        &self,
+        place: ScopedPlaceId,
+    ) -> BindingWithConstraintsIterator<'_, 'db> {
+        self.bindings_iterator(
+            &self.reachable_definitions[place].bindings,
+            BoundnessAnalysis::AssumeBound,
+        )
     }
 
     pub(crate) fn eager_snapshot(
@@ -409,50 +444,64 @@ impl<'db> UseDefMap<'db> {
             Some(EagerSnapshot::Constraint(constraint)) => {
                 EagerSnapshotResult::FoundConstraint(*constraint)
             }
-            Some(EagerSnapshot::Bindings(bindings)) => {
-                EagerSnapshotResult::FoundBindings(self.bindings_iterator(bindings))
-            }
+            Some(EagerSnapshot::Bindings(bindings)) => EagerSnapshotResult::FoundBindings(
+                self.bindings_iterator(bindings, BoundnessAnalysis::BasedOnUnboundVisibility),
+            ),
             None => EagerSnapshotResult::NotFound,
         }
     }
 
-    pub(crate) fn bindings_at_declaration(
+    pub(crate) fn bindings_at_definition(
         &self,
-        declaration: Definition<'db>,
+        definition: Definition<'db>,
     ) -> BindingWithConstraintsIterator<'_, 'db> {
-        self.bindings_iterator(&self.bindings_by_declaration[&declaration])
+        self.bindings_iterator(
+            &self.bindings_by_definition[&definition],
+            BoundnessAnalysis::BasedOnUnboundVisibility,
+        )
     }
 
     pub(crate) fn declarations_at_binding(
         &self,
         binding: Definition<'db>,
     ) -> DeclarationsIterator<'_, 'db> {
-        self.declarations_iterator(&self.declarations_by_binding[&binding])
+        self.declarations_iterator(
+            &self.declarations_by_binding[&binding],
+            BoundnessAnalysis::BasedOnUnboundVisibility,
+        )
     }
 
-    pub(crate) fn public_declarations<'map>(
+    pub(crate) fn end_of_scope_declarations<'map>(
         &'map self,
         place: ScopedPlaceId,
     ) -> DeclarationsIterator<'map, 'db> {
-        let declarations = self.public_places[place].declarations();
-        self.declarations_iterator(declarations)
+        let declarations = self.end_of_scope_places[place].declarations();
+        self.declarations_iterator(declarations, BoundnessAnalysis::BasedOnUnboundVisibility)
     }
 
-    pub(crate) fn all_public_declarations<'map>(
+    pub(crate) fn all_reachable_declarations(
+        &self,
+        place: ScopedPlaceId,
+    ) -> DeclarationsIterator<'_, 'db> {
+        let declarations = &self.reachable_definitions[place].declarations;
+        self.declarations_iterator(declarations, BoundnessAnalysis::AssumeBound)
+    }
+
+    pub(crate) fn all_end_of_scope_declarations<'map>(
         &'map self,
     ) -> impl Iterator<Item = (ScopedPlaceId, DeclarationsIterator<'map, 'db>)> + 'map {
-        (0..self.public_places.len())
+        (0..self.end_of_scope_places.len())
             .map(ScopedPlaceId::from_usize)
-            .map(|place_id| (place_id, self.public_declarations(place_id)))
+            .map(|place_id| (place_id, self.end_of_scope_declarations(place_id)))
     }
 
-    pub(crate) fn all_public_bindings<'map>(
+    pub(crate) fn all_end_of_scope_bindings<'map>(
         &'map self,
     ) -> impl Iterator<Item = (ScopedPlaceId, BindingWithConstraintsIterator<'map, 'db>)> + 'map
     {
-        (0..self.public_places.len())
+        (0..self.end_of_scope_places.len())
             .map(ScopedPlaceId::from_usize)
-            .map(|place_id| (place_id, self.public_bindings(place_id)))
+            .map(|place_id| (place_id, self.end_of_scope_bindings(place_id)))
     }
 
     /// This function is intended to be called only once inside `TypeInferenceBuilder::infer_function_body`.
@@ -463,6 +512,18 @@ impl<'db> UseDefMap<'db> {
             .is_always_false()
     }
 
+    pub(crate) fn is_declaration_reachable(
+        &self,
+        db: &dyn crate::Db,
+        declaration: &DeclarationWithConstraint<'db>,
+    ) -> Truthiness {
+        self.reachability_constraints.evaluate(
+            db,
+            &self.predicates,
+            declaration.reachability_constraint,
+        )
+    }
+
     pub(crate) fn is_binding_reachable(
         &self,
         db: &dyn crate::Db,
@@ -478,12 +539,14 @@ impl<'db> UseDefMap<'db> {
     fn bindings_iterator<'map>(
         &'map self,
         bindings: &'map Bindings,
+        boundness_analysis: BoundnessAnalysis,
     ) -> BindingWithConstraintsIterator<'map, 'db> {
         BindingWithConstraintsIterator {
             all_definitions: &self.all_definitions,
             predicates: &self.predicates,
             narrowing_constraints: &self.narrowing_constraints,
             reachability_constraints: &self.reachability_constraints,
+            boundness_analysis,
             inner: bindings.iter(),
         }
     }
@@ -491,11 +554,13 @@ impl<'db> UseDefMap<'db> {
     fn declarations_iterator<'map>(
         &'map self,
         declarations: &'map Declarations,
+        boundness_analysis: BoundnessAnalysis,
     ) -> DeclarationsIterator<'map, 'db> {
         DeclarationsIterator {
             all_definitions: &self.all_definitions,
             predicates: &self.predicates,
             reachability_constraints: &self.reachability_constraints,
+            boundness_analysis,
             inner: declarations.iter(),
         }
     }
@@ -510,9 +575,10 @@ impl<'db> UseDefMap<'db> {
 ///
 /// There is a unique ID for each distinct [`EagerSnapshotKey`] in the file.
 #[newtype_index]
+#[derive(get_size2::GetSize)]
 pub(crate) struct ScopedEagerSnapshotId;
 
-#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq, get_size2::GetSize)]
 pub(crate) struct EagerSnapshotKey {
     /// The enclosing scope containing the bindings
     pub(crate) enclosing_scope: FileScopeId,
@@ -531,6 +597,7 @@ pub(crate) struct BindingWithConstraintsIterator<'map, 'db> {
     pub(crate) predicates: &'map Predicates<'db>,
     pub(crate) narrowing_constraints: &'map NarrowingConstraints,
     pub(crate) reachability_constraints: &'map ReachabilityConstraints,
+    pub(crate) boundness_analysis: BoundnessAnalysis,
     inner: LiveBindingsIterator<'map>,
 }
 
@@ -611,6 +678,7 @@ pub(crate) struct DeclarationsIterator<'map, 'db> {
     all_definitions: &'map IndexVec<ScopedDefinitionId, DefinitionState<'db>>,
     pub(crate) predicates: &'map Predicates<'db>,
     pub(crate) reachability_constraints: &'map ReachabilityConstraints,
+    pub(crate) boundness_analysis: BoundnessAnalysis,
     inner: LiveDeclarationsIterator<'map>,
 }
 
@@ -639,6 +707,12 @@ impl<'db> Iterator for DeclarationsIterator<'_, 'db> {
 
 impl std::iter::FusedIterator for DeclarationsIterator<'_, '_> {}
 
+#[derive(Debug, PartialEq, Eq, salsa::Update, get_size2::GetSize)]
+struct ReachableDefinitions {
+    bindings: Bindings,
+    declarations: Declarations,
+}
+
 /// A snapshot of the definitions and constraints state at a particular point in control flow.
 #[derive(Clone, Debug)]
 pub(super) struct FlowSnapshot {
@@ -648,7 +722,7 @@ pub(super) struct FlowSnapshot {
 
 #[derive(Debug)]
 pub(super) struct UseDefMapBuilder<'db> {
-    /// Append-only array of [`Definition`].
+    /// Append-only array of [`DefinitionState`].
     all_definitions: IndexVec<ScopedDefinitionId, DefinitionState<'db>>,
 
     /// Builder of predicates.
@@ -673,12 +747,15 @@ pub(super) struct UseDefMapBuilder<'db> {
     /// Live declarations for each so-far-recorded binding.
     declarations_by_binding: FxHashMap<Definition<'db>, Declarations>,
 
-    /// Live bindings for each so-far-recorded declaration.
-    bindings_by_declaration: FxHashMap<Definition<'db>, Bindings>,
+    /// Live bindings for each so-far-recorded definition.
+    bindings_by_definition: FxHashMap<Definition<'db>, Bindings>,
 
     /// Currently live bindings and declarations for each place.
     place_states: IndexVec<ScopedPlaceId, PlaceState>,
 
+    /// All potentially reachable bindings and declarations, for each place.
+    reachable_definitions: IndexVec<ScopedPlaceId, ReachableDefinitions>,
+
     /// Snapshots of place states in this scope that can be used to resolve a reference in a
     /// nested eager scope.
     eager_snapshots: EagerSnapshots,
@@ -698,8 +775,9 @@ impl<'db> UseDefMapBuilder<'db> {
             reachability: ScopedReachabilityConstraintId::ALWAYS_TRUE,
             node_reachability: FxHashMap::default(),
             declarations_by_binding: FxHashMap::default(),
-            bindings_by_declaration: FxHashMap::default(),
+            bindings_by_definition: FxHashMap::default(),
             place_states: IndexVec::new(),
+            reachable_definitions: IndexVec::new(),
             eager_snapshots: EagerSnapshots::default(),
             is_class_scope,
         }
@@ -720,6 +798,11 @@ impl<'db> UseDefMapBuilder<'db> {
             .place_states
             .push(PlaceState::undefined(self.reachability));
         debug_assert_eq!(place, new_place);
+        let new_place = self.reachable_definitions.push(ReachableDefinitions {
+            bindings: Bindings::unbound(self.reachability),
+            declarations: Declarations::undeclared(self.reachability),
+        });
+        debug_assert_eq!(place, new_place);
     }
 
     pub(super) fn record_binding(
@@ -728,6 +811,9 @@ impl<'db> UseDefMapBuilder<'db> {
         binding: Definition<'db>,
         is_place_name: bool,
     ) {
+        self.bindings_by_definition
+            .insert(binding, self.place_states[place].bindings().clone());
+
         let def_id = self.all_definitions.push(DefinitionState::Defined(binding));
         let place_state = &mut self.place_states[place];
         self.declarations_by_binding
@@ -738,13 +824,35 @@ impl<'db> UseDefMapBuilder<'db> {
             self.is_class_scope,
             is_place_name,
         );
+
+        self.reachable_definitions[place].bindings.record_binding(
+            def_id,
+            self.reachability,
+            self.is_class_scope,
+            is_place_name,
+            PreviousDefinitions::AreKept,
+        );
     }
 
-    pub(super) fn add_predicate(&mut self, predicate: Predicate<'db>) -> ScopedPredicateId {
-        self.predicates.add_predicate(predicate)
+    pub(super) fn add_predicate(
+        &mut self,
+        predicate: PredicateOrLiteral<'db>,
+    ) -> ScopedPredicateId {
+        match predicate {
+            PredicateOrLiteral::Predicate(predicate) => self.predicates.add_predicate(predicate),
+            PredicateOrLiteral::Literal(true) => ScopedPredicateId::ALWAYS_TRUE,
+            PredicateOrLiteral::Literal(false) => ScopedPredicateId::ALWAYS_FALSE,
+        }
     }
 
     pub(super) fn record_narrowing_constraint(&mut self, predicate: ScopedPredicateId) {
+        if predicate == ScopedPredicateId::ALWAYS_TRUE
+            || predicate == ScopedPredicateId::ALWAYS_FALSE
+        {
+            // No need to record a narrowing constraint for `True` or `False`.
+            return;
+        }
+
         let narrowing_constraint = predicate.into();
         for state in &mut self.place_states {
             state
@@ -764,7 +872,7 @@ impl<'db> UseDefMapBuilder<'db> {
     /// This method exists solely for handling `*`-import reachability constraints.
     ///
     /// The reason why we add reachability constraints for [`Definition`]s created by `*` imports
-    /// is laid out in the doc-comment for [`StarImportPlaceholderPredicate`]. But treating these
+    /// is laid out in the doc-comment for `StarImportPlaceholderPredicate`. But treating these
     /// reachability constraints in the use-def map the same way as all other reachability constraints
     /// was shown to lead to [significant regressions] for small codebases where typeshed
     /// dominates. (Although `*` imports are not common generally, they are used in several
@@ -792,12 +900,10 @@ impl<'db> UseDefMapBuilder<'db> {
     /// [significant regressions]: https://github.com/astral-sh/ruff/pull/17286#issuecomment-2786755746
     pub(super) fn record_and_negate_star_import_reachability_constraint(
         &mut self,
-        star_import: StarImportPlaceholderPredicate<'db>,
+        reachability_id: ScopedReachabilityConstraintId,
         symbol: ScopedPlaceId,
         pre_definition_state: PlaceState,
     ) {
-        let predicate_id = self.add_predicate(star_import.into());
-        let reachability_id = self.reachability_constraints.add_atom(predicate_id);
         let negated_reachability_id = self
             .reachability_constraints
             .add_not_constraint(reachability_id);
@@ -842,9 +948,13 @@ impl<'db> UseDefMapBuilder<'db> {
             .all_definitions
             .push(DefinitionState::Defined(declaration));
         let place_state = &mut self.place_states[place];
-        self.bindings_by_declaration
+        self.bindings_by_definition
             .insert(declaration, place_state.bindings().clone());
         place_state.record_declaration(def_id, self.reachability);
+
+        self.reachable_definitions[place]
+            .declarations
+            .record_declaration(def_id, self.reachability, PreviousDefinitions::AreKept);
     }
 
     pub(super) fn record_declaration_and_binding(
@@ -866,6 +976,17 @@ impl<'db> UseDefMapBuilder<'db> {
             self.is_class_scope,
             is_place_name,
         );
+
+        self.reachable_definitions[place]
+            .declarations
+            .record_declaration(def_id, self.reachability, PreviousDefinitions::AreKept);
+        self.reachable_definitions[place].bindings.record_binding(
+            def_id,
+            self.reachability,
+            self.is_class_scope,
+            is_place_name,
+            PreviousDefinitions::AreKept,
+        );
     }
 
     pub(super) fn delete_binding(&mut self, place: ScopedPlaceId, is_place_name: bool) {
@@ -1000,10 +1121,11 @@ impl<'db> UseDefMapBuilder<'db> {
     pub(super) fn finish(mut self) -> UseDefMap<'db> {
         self.all_definitions.shrink_to_fit();
         self.place_states.shrink_to_fit();
+        self.reachable_definitions.shrink_to_fit();
         self.bindings_by_use.shrink_to_fit();
         self.node_reachability.shrink_to_fit();
         self.declarations_by_binding.shrink_to_fit();
-        self.bindings_by_declaration.shrink_to_fit();
+        self.bindings_by_definition.shrink_to_fit();
         self.eager_snapshots.shrink_to_fit();
 
         UseDefMap {
@@ -1013,9 +1135,10 @@ impl<'db> UseDefMapBuilder<'db> {
             reachability_constraints: self.reachability_constraints.build(),
             bindings_by_use: self.bindings_by_use,
             node_reachability: self.node_reachability,
-            public_places: self.place_states,
+            end_of_scope_places: self.place_states,
+            reachable_definitions: self.reachable_definitions,
             declarations_by_binding: self.declarations_by_binding,
-            bindings_by_declaration: self.bindings_by_declaration,
+            bindings_by_definition: self.bindings_by_definition,
             eager_snapshots: self.eager_snapshots,
             end_of_scope_reachability: self.reachability,
         }
