@@ -33,7 +33,7 @@ impl BackgroundRequestHandler for WorkspaceDiagnosticRequestHandler {
         let index = snapshot.index();
 
         if !index.global_settings().diagnostic_mode().is_workspace() {
-            tracing::debug!("Workspace diagnostics is disabled; returning empty report");
+            tracing::trace!("Workspace diagnostics is disabled; returning empty report");
             return Ok(WorkspaceDiagnosticReportResult::Report(
                 WorkspaceDiagnosticReport { items: vec![] },
             ));
@@ -65,7 +65,7 @@ impl BackgroundRequestHandler for WorkspaceDiagnosticRequestHandler {
                 let version = index
                     .key_from_url(url.clone())
                     .ok()
-                    .and_then(|key| index.make_document_ref(&key))
+                    .and_then(|key| index.make_document_ref(key).ok())
                     .map(|doc| i64::from(doc.version()));
 
                 // Convert diagnostics to LSP format
